@@ -44,11 +44,16 @@ typedef struct {
         gchar *weather_code;
         WeatherInfo *weather_info;
         guint weather_timeout;
+        guint weather_retry_time;
 
 	TempUnit temperature_unit;
 	SpeedUnit speed_unit;
 } ClockLocationPrivate;
 
+#define WEATHER_TIMEOUT_BASE 30
+#define WEATHER_TIMEOUT_MAX  1800
+#define WEATHER_EMPTY_CODE   "-"
+
 enum {
 	WEATHER_UPDATED,
 	SET_CURRENT,
@@ -63,6 +68,7 @@ static void clock_location_unset_tz (ClockLocation *this);
 static void setup_weather_updates (ClockLocation *loc);
 static void add_to_network_monitor (ClockLocation *loc);
 static void remove_from_network_monitor (ClockLocation *loc);
+static gchar* clock_location_get_valid_weather_code (const gchar *code);
 
 #define PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), CLOCK_LOCATION_TYPE, ClockLocationPrivate))
 
@@ -115,7 +121,7 @@ clock_location_new (const gchar *name, const gchar *timezone,
         priv->latitude = latitude;
         priv->longitude = longitude;
 
-        priv->weather_code = g_strdup (code);
+        priv->weather_code = clock_location_get_valid_weather_code(code);
 
 	if (prefs) {
 		priv->temperature_unit = prefs->temperature_unit;
@@ -486,7 +492,8 @@ free_make_current_data (gpointer data)
 }
 
 void
-clock_location_make_current (ClockLocation *loc, 
+clock_location_make_current (ClockLocation *loc,
+			     guint          transient_parent_xid,
                              GFunc          callback,
                              gpointer       data,
                              GDestroyNotify destroy)
@@ -525,13 +532,23 @@ clock_location_make_current (ClockLocation *loc,
 	mcdata->destroy = destroy;
 
         filename = g_build_filename (SYSTEM_ZONEINFODIR, priv->timezone, NULL);
-        set_system_timezone_async (filename, 
+        set_system_timezone_async (filename,
+				   transient_parent_xid,
                                    (GFunc)make_current_cb, 
 				   mcdata,
                                    free_make_current_data);
         g_free (filename);
 }
 
+static gchar *
+clock_location_get_valid_weather_code (const gchar *code)
+{
+	if (!code || code[0] == '\0')
+		return g_strdup (WEATHER_EMPTY_CODE);
+	else
+		return g_strdup (code);
+}
+
 const gchar *
 clock_location_get_weather_code (ClockLocation *loc)
 {
@@ -546,7 +563,7 @@ clock_location_set_weather_code (ClockLocation *loc, const gchar *code)
         ClockLocationPrivate *priv = PRIVATE (loc);
 
 	g_free (priv->weather_code);
-	priv->weather_code = g_strdup (code);
+	priv->weather_code = clock_location_get_valid_weather_code (code);
 
 	setup_weather_updates (loc);
 }
@@ -559,12 +576,44 @@ clock_location_get_weather_info (ClockLocation *loc)
 	return priv->weather_info;
 }
 
+static gboolean update_weather_info (gpointer data);
+
+static void
+set_weather_update_timeout (ClockLocation *loc)
+{
+	ClockLocationPrivate *priv = PRIVATE (loc);
+	guint timeout;
+
+	if (!weather_info_network_error (priv->weather_info)) {
+		/* The last update succeeded; set the next update to
+		 * happen in half an hour, and reset the retry timer.
+		 */
+		timeout = WEATHER_TIMEOUT_MAX;
+		priv->weather_retry_time = WEATHER_TIMEOUT_BASE;
+	} else {
+		/* The last update failed; set the next update
+		 * according to the retry timer, and exponentially
+		 * back off the retry timer.
+		 */
+		timeout = priv->weather_retry_time;
+		priv->weather_retry_time *= 2;
+		if (priv->weather_retry_time > WEATHER_TIMEOUT_MAX)
+			priv->weather_retry_time = WEATHER_TIMEOUT_MAX;
+	}
+
+	if (priv->weather_timeout)
+		g_source_remove (priv->weather_timeout);
+	priv->weather_timeout =
+		g_timeout_add_seconds (timeout, update_weather_info, loc);
+}
+
 static void
 weather_info_updated (WeatherInfo *info, gpointer data)
 {
 	ClockLocation *loc = data;
 	ClockLocationPrivate *priv = PRIVATE (loc);
 
+	set_weather_update_timeout (loc);
 	g_signal_emit (loc, location_signals[WEATHER_UPDATED],
 		       0, priv->weather_info);
 }
@@ -621,7 +670,11 @@ update_weather_infos (void)
 	GList *l;
 
 	for (l = locations; l; l = l->next) {
-		update_weather_info (l->data);
+		ClockLocation *loc = l->data;
+		ClockLocationPrivate *priv = PRIVATE (loc);
+
+		priv->weather_retry_time = WEATHER_TIMEOUT_BASE;
+		update_weather_info (loc);
 	}
 }
 
@@ -765,7 +818,7 @@ setup_weather_updates (ClockLocation *loc)
 		priv->weather_timeout = 0;
 	}
 
-	if (!priv->weather_code || strcmp (priv->weather_code, "-") == 0)
+	if (!priv->weather_code || strcmp (priv->weather_code, WEATHER_EMPTY_CODE) == 0)
 		return;
 
 	dms = rad2dms (priv->latitude, priv->longitude);
@@ -775,8 +828,7 @@ setup_weather_updates (ClockLocation *loc)
 	priv->weather_info =
 		weather_info_new (wl, &prefs, weather_info_updated, loc);
 
-	priv->weather_timeout =
-		g_timeout_add_seconds (1800, update_weather_info, loc);
+	set_weather_update_timeout (loc);
 
 	weather_location_free (wl);
 	g_free (dms);
